/**
 * iTunes Remote
 * Copyright cwalstad 2004,2005
 */

package walstad.itunesremote.common.security;

import java.io.Serializable;
import org.bouncycastle.crypto.macs.HMac;

import walstad.itunesremote.common.Message;

/**
 * A AuthenticatedMessage object represents a Message object that
 * is encypted with the key derived from the password
 * and contains a timestamp to provide data-origin authentication. <br><br><br>
 * 
 * Here is an overview of the authentication protocol we will use. <br>
 * The authentication protocol is based on the ISO One-Pass Unilateral
 * Authentication with Cryptographic Check Functions.  It is designed
 * to authenticate the remote to the host computer on every command that is sent
 * in a manner that is very quick and efficient.  The host computer to remote
 * messages contain no authentication.  The mechanism makes use of a HMAC using
 * the SHA256 message digest and a key derived from the user supplied password. <br>
 * The authentication uses timestamps to prevent replay.  This was chosen over nonces
 * because it does not require maintaining any state and does not require an extra
 * challenge for every message sent from the remote to the host.  Te first message
 * in the protocol is a message from the host to the remote that serves to indicate
 * whether authentication is required and to loosely synchronize their clocks.
 * 
 * @author Chris Walstad
 *
 */
public class AuthenticatedMessage implements Serializable {
    
    // Plain text portion
    private long timeStamp;
    private String host;
    private Message msg;
    
    // HMAC portion
    private byte[] hash;
    
    public AuthenticatedMessage(String host, Message msg, HMac hmac,
            long timestamp) {
        this.host = host;
        this.msg = msg;
        timeStamp = timestamp;
        
        //do the hash stuff
        hash = new byte[hmac.getMacSize()];
        // now need to turn the timestamp, host, and msg into a stream of bytes
        byte[] in = contentsToBytes();
        // now we need to create the hash, we assum hmac is initialized with the key and such
        hmac.reset();
        hmac.update(in,0,in.length);
        hmac.doFinal(hash,0);
    }
    
    /**
     * Takes the timestamp, host name, and message and concatenates them (in that order)
     * into one byte array
     * @return the constructed byte array representation
     */
    private byte[] contentsToBytes() {
        byte[] msgBytes = Tools.objToBytes(msg);
        byte[] timeStampBytes = Tools.longToBytes(timeStamp);
        byte[] hostBytes = host.getBytes();
        byte[] in = new byte[msgBytes.length + timeStampBytes.length + hostBytes.length];
        int i = 0;
        for(int j = 0;j<timeStampBytes.length;i++,j++)
            in[i] = timeStampBytes[j];
        for(int j = 0;j<hostBytes.length;i++,j++)
            in[i] = hostBytes[j];
        for(int j=0;j<msgBytes.length;i++,j++)
            in[i] = msgBytes[j];
        return in;
    }
    
    public boolean verifyHMac(HMac hmac) {
        byte[] msgContents = contentsToBytes();
        byte[] check = new byte[hmac.getMacSize()];
        hmac.reset();        
        hmac.update(msgContents,0,msgContents.length);
        hmac.doFinal(check,0);
        return Tools.byteArraysEqual(hash,check);
    }
    
    public byte[] getHash() {
        return hash;
    }
    
    /**
     * @return Returns the host.
     */
    public String getHost() {
        return host;
    }
    /**
     * @return Returns the msg.
     */
    public Message getMsg() {
        return msg;
    }
    /**
     * @return Returns the timeStamp.
     */
    public long getTimeStamp() {
        return timeStamp;
    }
}
