/**
 * iTunes Remote
 * Copyright cwalstad 2004,2005
 */

package walstad.itunesremote.local;

import java.net.Socket;
import java.net.ServerSocket;
import java.net.SocketException;
import java.net.SocketTimeoutException;

import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.Serializable;

import walstad.itunesremote.common.Common;
import walstad.itunesremote.common.Debugger;
import walstad.itunesremote.common.Message;
import walstad.itunesremote.common.security.AuthenticatedMessage;
import walstad.itunesremote.common.security.InitMessage;


/**
 * Manages a connection with a remote device
 */
public class Connection {
	
	private Socket socket;
	private ObjectOutputStream os;
	private ObjectInputStream is;
	
	private volatile boolean keepRunning = true;
    private static final int TIME_DIFF = 15000;
	
	public Connection() {}
	
	
	public void end() {
		keepRunning = false;
	}
	
	public void run() {
		connect();
	}
	
	public boolean connect() {
		ServerSocket ss = null;
		try {
			ss = new ServerSocket(Common.CONNECTION_PORT);
			ss.setSoTimeout(100);
			while(socket == null && keepRunning) {
				try {
					socket = ss.accept();
				}catch(SocketTimeoutException e) {}
			}
			os = new ObjectOutputStream(socket.getOutputStream());
			is = new ObjectInputStream(socket.getInputStream());
		}catch(Exception e) {
			e.printStackTrace();
			try {ss.close();} catch(Exception e1) {e1.printStackTrace();}
			cleanup();
			return false;
		}
		try {ss.close();}catch(Exception e) {}
		sendInitMessage();
		return true;
	}
	
	
	/**
	 * Sends the InitMessage of iTunes.common.security
	 * This message informs the remote whether it needs
	 * to authenticate and gives it a timestamp to loosely
	 * synchronize its clock with the local clock
	 */
	private void sendInitMessage() {
	    Properties p = Main.getProperties();
	    InitMessage msg = new InitMessage(p.isPasswordRequired());
	    try {
	        send(msg);
	    }catch(IOException e) {
	        e.printStackTrace();
	    }
	}
	
	public synchronized void send(Serializable[] m) throws IOException {
		for(int i=0;i<m.length;i++) {
			os.writeObject(m[i]);
		}
		os.flush();
	}
	
	public synchronized void send(Serializable m) throws IOException {
		os.writeObject(m);
		os.flush();
	}
	
	/** 
	 * Read one message from the input stream and returns it
	 * 
	 * @return the message read
	 * 
	 * @throws IOException
	 * @throws SocketException
	 * @throws ClassNotFoundException
	 */
	public Message receive() throws IOException, SocketException,ClassNotFoundException {
	    Serializable o = (Serializable)is.readObject();
	    Properties p = Main.getProperties();
	    if(p.isPasswordRequired()) {
	        if(!(o instanceof AuthenticatedMessage)) {
	            Debugger.println("Rejected non-authenticated message");
	            sendInvalidHMacMessage(Common.REQUIRES_PASSWORD);
	            return null;
	        }
	        AuthenticatedMessage authMsg = (AuthenticatedMessage)o;
	        if(!authMsg.verifyHMac(Main.getHMac())) {
	            Debugger.println("Rejected a message with invalid HMAC");
	            sendInvalidHMacMessage(Common.BAD_PASSWORD);
	            return null;
	        }
	        // it has authenticated return the message
	        long curTime = System.currentTimeMillis();
	        if(authMsg.getTimeStamp() < curTime + TIME_DIFF && 
	                authMsg.getTimeStamp() > curTime - TIME_DIFF) {
	            return authMsg.getMsg();
	        }else {
	            Debugger.println("Rejected a message due to old timestamp");
	            return null;
	        }
	    }else {
	        if(o instanceof AuthenticatedMessage) {
	            AuthenticatedMessage authMsg = (AuthenticatedMessage)o;
	            return authMsg.getMsg();
	        }else if(o instanceof Message) {
	            Message msg = (Message)o;
	            return msg;
	        }else {
	            return null;
	        }
	    }
	}
	
	private void sendInvalidHMacMessage(int code) {
	    Message msg = new Message(code,null);
	    try {
	        os.writeObject(msg);
	    }catch(Exception e) {
	        e.printStackTrace();
	    }
	}
	
	protected void cleanup() {
		try { socket.close(); } catch(Exception e) {}
		try { os.close(); } catch(Exception e) {}
		try { is.close(); } catch(Exception e) {}
	}

}
