/**
 * iTunes Remote
 * Copyright cwalstad 2004,2005
 */

package walstad.itunesremote.local;


import java.io.IOException;

import walstad.itunesremote.common.*;

/**
 * Receives messages and responds
 * 
 * This is a daemon thread that will constantly wait for new
 * messages and execute commands appropriately
 *
 */
public class Receiver {

	private Player player;
	private Connection connection;
	
	private volatile boolean keepRunning = true;
	private Exception error = null;
	
	public Receiver(Player player,Connection connection) {
		this.player = player;
		this.connection = connection;
	}
	
	public void end() {
		keepRunning = false;
	}
	
	public boolean wasError() {
		return (error != null);
	}
	
	public Exception getError() {
		return error;
	}
	
	public void run() {
		while(keepRunning) {
			Message m = null;
			try {
				m = connection.receive();
			}catch(IOException e) {
				e.printStackTrace();
				error = e;
				break;
			}catch(ClassNotFoundException e) {
				e.printStackTrace();
				continue;
			}
			if(m == null)
			    continue;
			handleMessage(m);
		}
	}
	
	private void handleMessage(Message m) {
		int code = m.getCode();
		switch(code) {
		case Common.PLAY_ID:
			handlePlay(m);
			break;
		case Common.FAST_FORWARD_ID:
			handleFastForward(m);
			break;
		case Common.GET_CURRENT_TRACK_INFO_ID:
			handleGetCurrentTrackInfo(m);
			break;
		case Common.MUTE_ID:
			handleMute(m);
			break;
		case Common.NEXT_TRACK_ID:
			handleNextTrack(m);
			break;
		case Common.PLAY_PAUSE_ID:
			handlePlayPause(m);
			break;
		case Common.PREV_TRACK_ID:
			handlePrevTrack(m);
			break;
		case Common.QUIT_ID:
			handleQuit(m);
			break;
		case Common.RESUME_ID:
			handleResume(m);
			break;
		case Common.REWIND_ID:
			handleRewind(m);
			break;
		case Common.STOP_ID:
			handleStop(m);
			break;
		case Common.VOLUME_DOWN_ID:
			handleVolumeDown(m);
			break;
		case Common.VOLUME_UP_ID:
			handleVolumeUp(m);
			break;
		case Common.SEARCH_ALL_QUERY:
			handleSearchAllQuery(m);
			break;
		default:
			throw new UnsupportedOperationException("Invalid ID!!!");
		}
	}
	
	private int handlePlay(Message m) {
		return player.play((Track)m.getData());
	}
	
	private int handleSearchAllQuery(Message m) {
		SearchQuery sq = (SearchQuery)m.getData();
		if(sq == null) {
			Debugger.println("SearchQuery is null");
			return Common.UNEXPECTED_ERROR;
		}
		SearchResult sr = player.search(sq);
		Message ret = new Message(Common.SEARCH_RESULTS_HEADER,sr);
		try {
			connection.send(ret);
		}catch(IOException e) {
			e.printStackTrace();
			return Common.UNEXPECTED_ERROR;
		}
		return Common.SUCCESS;
	}
	
	private int handleVolumeUp(Message m) {
		return player.volumeUp();
	}
	
	private int handleVolumeDown(Message m) {
		return player.volumeDown();
	}
	
	private int handleStop(Message m) {
		return player.stop();
	}
	
	private int handleRewind(Message m) {
		return player.rewind();
	}
	
	private int handleResume(Message m) {
		return player.resume();
	}
	
	private int handleMute(Message m) {
		return player.mute();
	}
	
	private int handleQuit(Message m) {
		return player.quit();
	}
	
	private int handlePrevTrack(Message m) {
		return player.previousTrack();
	}
	
	private int handleNextTrack(Message m) {
		return player.nextTrack();
	}
	
	private int handlePlayPause(Message m) {
		return player.playPause();
	}
	
	private int handleFastForward(Message m) {
		return player.fastForward();
	}
	
	
	private int handleGetCurrentTrackInfo(Message m) {
		try {
			Message send = new Message(Common.GET_CURRENT_TRACK_INFO_ID,
					player.getCurrentTrack());
			connection.send(send);
		}catch(IOException e) {
			e.printStackTrace();
			return Common.UNEXPECTED_ERROR;
		}
		return Common.SUCCESS;
	}
	

}
