/**
 * iTunes Remote
 * Copyright cwalstad 2004,2005
 */

package walstad.itunesremote.remote;

import java.net.Socket;
import java.net.InetAddress;
import java.net.UnknownHostException;

import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;


import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.crypto.macs.HMac;
import org.bouncycastle.crypto.digests.SHA256Digest;

import walstad.itunesremote.common.Common;
import walstad.itunesremote.common.Debugger;
import walstad.itunesremote.common.Host;
import walstad.itunesremote.common.Message;
import walstad.itunesremote.common.security.AuthenticatedMessage;
import walstad.itunesremote.common.security.InitMessage;
import walstad.itunesremote.remote.gui.GuiGetUserPasswordDialog;




/**
 * Manages a connection with the computer
 */
public class Connection {
	
	private Socket socket;
	private ObjectOutputStream os;
	private ObjectInputStream is;
	
	private long timestampOffset = 0;
	
	private Host host;
	private boolean requirePassword;
	private HMac hmac;
	
	public Connection() {}
	
	public boolean connect(String addr) throws UnknownHostException{
		return connect(InetAddress.getByName(addr));
	}
	
	public boolean connect(Host host) {
	    this.host = host;
		return connect(host.getAddress());
	}
	
	private boolean connect(InetAddress addr) {
		try {
			socket = new Socket(addr,Common.CONNECTION_PORT);
			os = new ObjectOutputStream(socket.getOutputStream());
			is = new ObjectInputStream(socket.getInputStream());
		}catch(Exception e) {
			e.printStackTrace();
			cleanup();
			return false;
		}
		receiveInitMessage();
		return true;
	}
	
	private void receiveInitMessage() {
	    InitMessage initMsg = null;
	    try {
	        final int TIMEOUT = 5 * 1000;
	        socket.setSoTimeout(TIMEOUT);
	        initMsg = (InitMessage)is.readObject();
	        socket.setSoTimeout(0);
	    }catch(IOException e) {
	        Debugger.println("IOException reading initMessage");
	        e.printStackTrace();
	        return;
	    }catch(ClassNotFoundException e) {
	        Debugger.println("Class not found exception");
	        e.printStackTrace();
	        return;
	    }
	    long time = System.currentTimeMillis();
	    timestampOffset = initMsg.getTime() - time;
	    requirePassword = initMsg.passwordRequired();
	    String password = null;
	    if(requirePassword) {
	        if(host.isRememberPassword()) {
	            password = host.getPassword();
	        }else {
	            password = getUserPassword();
	            if(password == null) { // chose to cancel
	                Main.disconnect();
	            }
	        }
	        hmac = new HMac(new SHA256Digest());
	        hmac.init(new KeyParameter(walstad.itunesremote.common.security.Tools.stringToKey(password)));
	    }
	    
	}
	
	private String getUserPassword() {
	    GuiGetUserPasswordDialog dialog = new GuiGetUserPasswordDialog();
	    dialog.show();
	    while(!dialog.isDone()) {
	        try {
	            Thread.sleep(20);
	        }catch(InterruptedException e) {
	            e.printStackTrace();
	            return null;
	        }
	    }
	    String password = dialog.getPassword();
	    boolean rememberPassword = dialog.rememberPassword();
	    if(rememberPassword) {
	        host.setPassword(password);
	        host.setRememberPassword(true);
	        SavedHosts.writeSavedHots();
	    }
	    return password;
	}
	
	public void send(Message[] m) throws IOException {  
		for(int i=0;i<m.length;i++) {
			send(m[i]);
		}
	}
	
	public void send(Message m) throws IOException {
	    if(!requirePassword) {
	        os.writeObject(m);
	    }else {
	        AuthenticatedMessage authMsg = new AuthenticatedMessage(host.getHostName(),
	                m,hmac, System.currentTimeMillis() + timestampOffset);
	        os.writeObject(authMsg);
	    }
		os.flush();
	}
	
	public Message receive() throws IOException,ClassNotFoundException {
		return (Message)is.readObject();
	}
	
	protected void cleanup() {
		try { socket.close(); } catch(Exception e) {}
		try { os.close(); } catch(Exception e) {}
		try { is.close(); } catch(Exception e) {}
	}

}